@@ -39,11 +39,14 @@ def get_nearby_lightning(tgf,lightnings:array,threshold:float=1):
     Returns:
         lightnings (array): A filtered array of lightnings
     """
+    # If hardware accelleration is not available use numpy instead
+    if not CUPY_AVAILABLE:cp = np
 
     # If we are given an array of TGFs
     if type(tgf) == array:
         # Create a list to output the lighning arrays for each event
-        lights = []
+        tgf = cp.asarray(tgf)
+        lights = cp.array([])
 
         # For all the events
         for T in tqdm(tgf,desc='Event'):
@@ -62,13 +65,14 @@ def get_nearby_lightning(tgf,lightnings:array,threshold:float=1):
         tgf_time = tgf.timestamp
 
         # Get all the timestamps
-        timestamps = lightnings.get_timestamps()
+        timestamps = cp.asarray(lightnings.get_timestamps())
 
         # find the indices where the timedifference is less than threshold
-        idx = [i for i,time in enumerate(timestamps) if abs(time - tgf_time) < threshold]
-
+        # idx = [i for i,time in enumerate(timestamps) if abs(time - tgf_time) < threshold]
+        idx = cp.asarray((abs(timestamps -tgf_time.to_value(format="unix")) < threshold.to_value(format="sec")).nonzero())
+        idx = idx.get()[0]
         # Get the appropriate subarray
-        return array(lightnings[idx])
+        return array(lightnings[idx.to_list()])
 
     # if it is not of type event of array then raise an error
     else:
@@ -204,7 +208,7 @@ def correct_bit(x:int, MEAN:float, BITS:int=48):
 
 
 # Find pairs
-def find_pairs(diffs,MEAN,STD):
+def find_pairs(diffs,MEAN,STD,BITS:int=50):
     """Helper function to find the pairs of bit flips occuring in a timestring, Use invert_flips instead!
     """
     candidates  = np.array([])
@@ -212,20 +216,22 @@ def find_pairs(diffs,MEAN,STD):
     round_msb = lambda data,BITS: get_msb((3*abs(data)).astype(int) >> 1,BITS)*np.sign(data)
 
     pairs = []    
-    for i,d in enumerate(round_msb(diffs,50)):
+    for i,d in enumerate(tqdm(round_msb(diffs,BITS),desc='Bit Flipper')):
         # If this is a candidate for a thing 
-        if d < -MEAN - 3*STD:
+        if abs(d) > MEAN + 2*STD:
             candidates = np.append(candidates,[d])
             idxs       = np.append(idxs      ,[i])
-        
+
         if len(candidates) > 0:
             # candidates += d
             idx = np.where(abs(candidates + d) == 0)[0]
             if len(idx) > 0:
-                index = idx[0]
-                pairs.append((idxs[index],i))
-                candidates = np.delete(candidates,[index,int(-1)],axis=0)
-                idxs       = np.delete(idxs      ,[index,int(-1)],axis=0)
+                for index in idx:
+                    if abs(i-idxs[index]) < 5000:
+                        pairs.append((idxs[index],i))
+                        candidates = np.delete(candidates,[index,int(-1)],axis=0)
+                        idxs       = np.delete(idxs      ,[index,int(-1)],axis=0)
+                        break
 
     return np.array(pairs)
 
@@ -248,8 +254,8 @@ def invert_flips(timestamp:np.array,BITS:int=NONVETO_STRUCT['stimestamp']):
     STD  = np.std(abs(timestamp_deltas))
 
     # Identify the pairs of points where you get bit flips
-    pairs = find_pairs(timestamp_deltas,MEAN,STD)
-    
+    pairs = find_pairs(timestamp_deltas,MEAN,STD,BITS)
+
     # For each bit flip region
     for pair in pairs:
         ADD = 0
@@ -303,8 +309,7 @@ def get_bits(start:int,length:int,string,STUPID:bool=False):
 
     return digit_sum
 
-# Create a dictionary of orbits from a file
-def get_dict(filename:str,struct=ORBIT_STRUCT,condition:str=None,MAX=None,STUPID:bool=False,VERIFY=False,threshold=5e-5):
+def get_dict(filename:str,struct=ORBIT_STRUCT,condition:str=None,MAX=None,STUPID:bool=False,VERIFY=False,threshold=5e-5,LAST:int=None,min_streak:int=-1):
     """Decode the data of a buffer with a given structure into a dictionary
 
     Args:
@@ -323,15 +328,13 @@ def get_dict(filename:str,struct=ORBIT_STRUCT,condition:str=None,MAX=None,STUPID
     file = open(filename,'rb')  # Open the file in read binary mode
     raw = file.read()           # Read all the file
     file.close()                # Close the file
-
-    # Initialize the dictionary
-    data = dict(zip(struct.keys(),[np.array(list()) for _ in range(len(ORBIT_STRUCT.keys()))]))
-
+    
     # Number of bytes per line
     bytes_per_line  = sum(list(struct.values()))//8
     length          = len(raw)//bytes_per_line
     if MAX is None or MAX > length: MAX = length
-
+    if LAST is not None: MAX = LAST
+    
     # Check if VERIFICATION can occur
     if VERIFY:
         # If you can't correct then don't
@@ -345,10 +348,19 @@ def get_dict(filename:str,struct=ORBIT_STRUCT,condition:str=None,MAX=None,STUPID
             THRESHOLD *= threshold
         else:
             THRESHOLD = threshold
-
+    
     # Current byte index in the file
-    curr = 0
-    with tqdm(total=MAX,desc='Line: ') as pbar:
+    curr = 0 if LAST is None else len(raw) - LAST * bytes_per_line
+
+    # Initialize the dictionary
+    data = dict(zip(struct.keys(),[ [0]*MAX for _ in range(len(struct.keys()))]))
+
+    with tqdm(total=MAX,desc='Line: ', miniters=10) as pbar:
+    
+        # Counter for streaks of zero
+        zero_streak = 0
+        last_curr   = 0
+        last_i      = 0
         # Index of line
         i = 0
         while i < MAX:
@@ -364,34 +376,77 @@ def get_dict(filename:str,struct=ORBIT_STRUCT,condition:str=None,MAX=None,STUPID
 
             # Keep track of the number of bits read
             bits_read = 0
-
             # If not create an orbit
             for name,length in struct.items():
-                data[name] = np.append(data[name],[get_bits(bits_read,length,event,STUPID=STUPID)])
+                # data[name].append(get_bits(bits_read,length,event,STUPID=STUPID))
+                data[name][i] = get_bits(bits_read,length,event,STUPID=STUPID)
                 bits_read += length
 
             # Verify the datum makes sense
             if VERIFY:
-                # If there are more than two datapoints in the timestamp
-                if len(data['stimestamp'])>=2:
-                    # If the difference between the last two timestmaps is absurd
-                    if abs(data['stimestamp'][-1] - data['stimestamp'][-2]) > THRESHOLD:# or \
-                       #(data['stimestamp'][-1] - data['stimestamp'][-2] < 0) and (abs(data['stimestamp'][-2] - data['stimestamp'][-1]) < THRESHOLD/20):
-                        # remove the previous datapoint
-                        for key in data.keys():
-                            data[key] = np.delete(data[key],-1)
+                # Update the zero streak counter
+                if min_streak >= 0:
+                    if data['channel'][i] == 0: zero_streak += 1
+                    elif zero_streak > min_streak:
+                        tqdm.write(f'ZERO COUNTS TRIGGERED AT {i} with zero_streak = {zero_streak}, curr = {curr}')
+                        # i          -= zero_streak - 1
+                        # curr       -= -2 + bytes_per_line * (zero_streak)
+                        i           = last_i - 10
+                        curr        = last_curr + 1 - 10*bytes_per_line
+                        last_curr   = curr
+                        update      = 0.1
+                        pbar.colour = '#ffff00'
+                        pbar.total += 2
+                        zero_streak = 0
+                        tqdm.write(f'\t     RETRYING AT {i} with zero_streak = {zero_streak}, curr = {curr}')
+                        continue
+                    else: 
+                        zero_streak = 0
+                        last_curr   = curr
+                        last_i      = i
 
+                # If there are more than two datapoints in the timestamp
+                if i>=1:#len(data['stimestamp'])>=2:
+                    # If the difference between the last two timestmaps is absurd
+                    if (data['stimestamp'][i] - data['stimestamp'][i-1] > THRESHOLD) or (THRESHOLD*0.05 < data['stimestamp'][i-1] -  data['stimestamp'][i] < THRESHOLD*0.3):# or data['stimestamp'][i] > 2*THRESHOLD:# or (data['adc_counts'][-1] <= 3):
+                    # if data['stimestamp'][-1] - data['stimestamp'][-2] > THRESHOLD:# or (data['adc_counts'][-1] <= 3):
+                        # tqdm.write(f"{curr}: {data['stimestamp'][-1]} - {data['stimestamp'][-2]} = {abs(data['stimestamp'][-1] - data['stimestamp'][-2])} > {THRESHOLD} {abs(data['stimestamp'][-1] - data['stimestamp'][-2]) > THRESHOLD}")
+                        # remove the previous datapoint
+                        # for key in data.keys():
+                        #     # print(data[key][-1])
+                        #     data[key] = data[key][:-1]
                         # Move forward by two bytes
-                        curr   -= bytes_per_line - 2
+                        curr   -= - 1 + bytes_per_line
                         i      -= 1
-                        update  = 0
+                        update  = 1.1
+                        pbar.colour = '#ff0000'
+                        pbar.total += 1
+
+                # elif data['stimestamp'][-2] - data['stimestamp'][-1] > THRESHOLD:
+
+                elif i==0 and (data['stimestamp'][0] > THRESHOLD or data['adc_counts'][0] <= 3) :
+                # elif len(data['stimestamp'])==1 and (data['stimestamp'][0] > THRESHOLD or data['adc_counts'][0] <= 3) :
+                    # for key in data.keys():
+                    #     # print(data[key][-1])
+                    #     data[key] = data[key][:-1]
+                    # Move forward by two bytes
+                    # tqdm.write(f"{curr}: {data['stimestamp'][-1]} - {data['stimestamp'][-2]} = {abs(data['stimestamp'][-1] - data['stimestamp'][-2])} > {THRESHOLD} \n\t{data['adc_counts'][0]}")
+                    curr   -= - 1 + bytes_per_line
+                    i      -= 1
+                    update  = 1.1
+                    pbar.colour = '#ff0000'
+                    pbar.total += 1
+                    
 
             # Update reader position
             curr    += bytes_per_line
             i       += 1
-            pbar.update(update)
+            # print(curr,update)
+            pbar.update(np.floor(update))
+            if update == 1: pbar.colour = '#0000ff'
 
-        
+    for name, value in data.items():
+        data[name] = np.array(value) 
     # If you want to filter, then apply the filter to the loaded data directly
     if condition is not None:
         try:
@@ -399,24 +454,25 @@ def get_dict(filename:str,struct=ORBIT_STRUCT,condition:str=None,MAX=None,STUPID
             data    = dict(zip(struct.keys(),[arr[idx] for arr in data.values()]))
         except:
             print(bcolors.WARNING+'WARNING!' + bcolors.ENDC +' Condition ' + condition + ' is not valid for the dataset you requested. The data returned will not be filtered')
-
     # Specific loading changes
     if 'temperature' in struct.keys():
-        data['temperature'] -= 55
+        data['temperature'] = [i - 55 for i in data['temperature']]
         
 
     # If we can do a bit flip verification perform it
     if VERIFY:
-        # Split to channels
-        channels, cnt = split_channels(data,struct)
-        
-        # Apply correction to each channel
-        for channel in channels: channel['stimestamp'] = invert_flips(channel['stimestamp'],struct['stimestamp'])
+        # # Split to channels
+        # channels, cnt = split_channels(data,struct)
 
-        # Put it back together
-        for i, channel in enumerate(channels):
-            for j, time in enumerate(channel['stimestamp']):
-                data['stimestamp'][cnt[i][j]] = time
+        # # Apply correction to each channel
+        # for channel in channels: channel['stimestamp'] = invert_flips(channel['stimestamp'],struct['stimestamp'])
+
+        # # Put it back together
+        # for i, channel in enumerate(channels):
+        #     for j, time in enumerate(channel['stimestamp']):
+        #         data['stimestamp'][cnt[i][j]] = time
+
+        data['stimestamp'] = invert_flips(data['stimestamp'],struct['stimestamp'])
     
     # Return the dictionary
     return data
@@ -651,7 +707,7 @@ def correct_time(data:dict,orbit:dict,key:str='rate0',TIME:int=20,RANGE_ORBIT=(0
 
 
 # Download a range of data based on some limit
-def download_range(url:str,token,limit:int=5000,VERBOSE:bool=False):
+def download_range(url:str,token,limit:int=5000,VERBOSE:bool=False,clear=clear):
     """Downloads a range of data given a url and a token from the NA servers. 
     Automatically handles large file sizes.
 
@@ -682,14 +738,15 @@ def download_range(url:str,token,limit:int=5000,VERBOSE:bool=False):
             cnt+=1
 
         # Do the REST stuff
-        rest = RestOperations(url+f'&limit={limit}&seq_nr=gte.{seq}', authType = 'token', token = token)
+        append = (f'&limit={limit}&seq_nr=gt.{seq}' if url[-1] != '?' else f'limit={limit}&seq_nr=gt.{seq}') if limit >= 0 else ''
+        rest = RestOperations(url+append, authType = 'token', token = token)
        
         # Download the data
         last_data   = rest.SendGetReq()
         data        += last_data
 
         # If there are no more data exit
-        if len(last_data) < limit or seq == max([datum['seq_nr'] for datum in data]):
+        if len(last_data) < limit or limit == -1: # or seq == max([datum['seq_nr'] for datum in data]):
             return data
         
         # Find the last sequence number
@@ -775,8 +832,8 @@ def download_time_delta(buffer:int = 1, start:str=None, end:str=None):
 
     Args:
         buffer (int, optional): The buffer number. Defaults to 1.
-        start (str, optional): String with iso date to start. Defaults to '2022-06-01T00:00:00'.
-        end (str, optional): String with iso date to end. Defaults to '2022-06-07T00:00:00'.
+        start (str, optional): String with iso date to start. Defaults to None.
+        end (str, optional): String with iso date to end. Defaults to None.
 
     Returns:
         data: list of dictionaries with the rows
@@ -860,7 +917,7 @@ def log_to_ascii(data,fileName:str=None):
     return full_text
 
 # Parse a logfile and obtain metadata
-def log_expand(filename:str=None,text:str=None):
+def log_expand(filename:str=None,text:str=None,remove_comments:bool=False):
     """Gets a logfile and decodes it to a list of commands. 
     If a text value is given then it decodes the text, if not, it then decodes the value from the filename
 
@@ -895,13 +952,14 @@ def log_expand(filename:str=None,text:str=None):
     if "SE0>" not in loglines[-1]: loglines.append("SE0>")
 
     # Decode the file
-    # Find the indices of hte command lines
-    commands_idx = [i for i,line in enumerate(loglines) if 'SE0>' in line]
-    
+    # Find the indices of the command lines
+    commands_idx = [i for i,line in enumerate(loglines) if ('SE0>' in line) and (('#' not in line) or not remove_comments)]
+
     # Collect the outputs of the commands
     decoded_log = [{
         'command':loglines[commands_idx[i]],
-        'output' :loglines[commands_idx[i]+1:commands_idx[i+1]]
+        'output' :loglines[commands_idx[i]+1:commands_idx[i+1]],
+        'index'  :commands_idx[i]
         } for i in range(len(commands_idx)-1)]
 
     # Return
@@ -1115,25 +1173,32 @@ def log_line_timestamp(logline:list,time:float=0):
         time (float): Time increment after the execution of the command
     """
 
-    splt = logline['command'].split(' ')                # Split the cmnd line (logline[0]) by the spaces 
-    if "SE0>\n" not in splt[0]:                         # If the cmnd is not the end of the log file: 
+    splt = logline['command'].split(' ')                            # Split the cmnd line (logline[0]) by the spaces 
+    if "SE0>\n" not in splt[0] and len(splt) > 1:                   # If the cmnd is not the end of the log file: 
         
-        if "read" in splt[1]:                           # If cmnd is read time
-            time = float (logline['output'][0].split(" ")[3])  # Replace time with the time read
+        try:
+            if "FAIL\n" in logline['output']:                       # if it was a payload cmnd and it failed (there will be delays)
+                if "txrx" in splt[1]:time + float(splt[4])/1000     # take the time a payload cmnd takes to be excuted 
 
-        if splt[1] == "delay":                          # If the cmnd is a delay
-            time = time + (float(splt[2])/1000)         # Add time delay to previous time
+            elif "read" in splt[1]:                                 # If cmnd is read time
+                time = float(logline['output'][0].split(" ")[3])   # Replace time with the time read
 
-        if splt[1] == "delayuntil":                     # If cmnd is delayuntil 
-            time = float(splt[2])                       # Replace time by new time
+            elif splt[1] == "delay":                                # If the cmnd is a delay
+                time = time + (float(splt[2])/1000)                 # Add time delay to previous time
 
-        if "FAIL\n" in logline['output']:               # if it was a payload cmnd and it failed (there will be delays)
-            time = time + float(splt[4])/1000                # take the time a payload cmnd takes to be excuted 
+            elif splt[1] == "delayuntil":                           # If cmnd is delayuntil 
+                time = float(splt[2])                               # Replace time by new time
+        
+        except ValueError:
+            print('Found weird command\n', logline)
+        
+        except IndexError:
+            print("Index Error in command\n",logline)
 
     return time
 
 # Now we can define an error correction pass
-def find_closest(x,array):
+def find_closest(x,array,return_idx:bool=False):
     """Helper function that returns the two closest values to x in a list
 
     Args:
@@ -1150,21 +1215,20 @@ def find_closest(x,array):
     prevmid = -float('inf')
 
     # Handle edge cases
-    if x <= array[s]: return (-float('inf'),array[s])
-    if x >= array[e]: return (array[e],float('inf'))
+    if x <= array[s]: return (-float('inf'),array[s]) if not return_idx else [(-float('inf'),array[s]),(-float('inf'),s)]
+    if x >= array[e]: return (array[e],float('inf'))  if not return_idx else [(array[e],float('inf')), (e,float('inf'))]
 
     # Flag that will update condition to exit
-    found = False
     while prevmid != mid:
-        if x == array[mid]: return (array[mid-1],array[mid])
+        if x == array[mid]: return (array[mid-1],array[mid]) if not return_idx else [(array[mid-1],array[mid]),(mid-1,mid)]
         if x < array[mid]:  e = mid
         if x > array[mid]:  s = mid
 
         prevmid = mid
         mid     = (s+e)//2
 
-    if (s==e): return (array[s],array[s+1])
-    return (array[s],array[e])
+    if (s==e): return (array[s],array[s+1]) if not return_idx else [(array[s],array[s+1]),(s,s+1)]
+    return (array[s],array[e]) if not return_idx else [(array[s],array[e]),(s,e)]
 
 
 
@@ -1204,7 +1268,7 @@ def reorder_log(logfile:list):
     return logfile
 
 
-def log_with_timestamp(logfile:list):                        
+def log_with_timestamp(logfile:list,reorder=True):                        
     """Given a log file from the function log_expand
     return a list of log lines with the time increment of each line
 
@@ -1221,10 +1285,11 @@ def log_with_timestamp(logfile:list):
         log_timestamp_list.append ({
             'command':      logline['command'],
             'output':       logline['output'],
+            'index':        logline['index'],
             'timestamp':    time
         })
 
-    return reorder_log(log_timestamp_list)
+    return reorder_log(log_timestamp_list) if reorder else log_timestamp_list
 
 
 def get_cmd_number(cmdline:str):
@@ -1240,10 +1305,15 @@ def get_cmd_number(cmdline:str):
     value = -1                          # Initialize the value to be -1 so that if the function faces commands that are not in the CMND_LIST it will give it a -1 value
     split_cmdline = cmdline.split(" ")  # Split the cmd by the spaces in between
     if len(split_cmdline) > 1 and 'txrx' in split_cmdline[1]: # If the cmd is a payload cmd, is has txrx
-        if float(split_cmdline[2]) == 4:                      # If the cmd belongs to emergency poweroff then the message is not needed (message is split_cmdline[5])
-            value = CMND_LIST[split_cmdline[2]][split_cmdline[3]]['number_in_graph'] # Grab the cmd number from the CMND_LIST based on the node and port of the cmd ([split_cmdline[2]][split_cmdline[3]])
-        else:
-            value = CMND_LIST[split_cmdline[2]][split_cmdline[3]][split_cmdline[5]]['number_in_graph'] #If not an emergency poweroff use node, port, and message to find the cmd in the CMND_LIST
+        try:
+            if float(split_cmdline[2]) == 4:                      # If the cmd belongs to emergency poweroff then the message is not needed (message is split_cmdline[5])
+                value = CMND_LIST[split_cmdline[2]][split_cmdline[3].strip()]['number_in_graph'] # Grab the cmd number from the CMND_LIST based on the node and port of the cmd ([split_cmdline[2]][split_cmdline[3]])
+            else:
+                value = CMND_LIST[split_cmdline[2]][split_cmdline[3].strip()][split_cmdline[5].strip()]['number_in_graph'] #If not an emergency poweroff use node, port, and message to find the cmd in the CMND_LIST
+        except KeyError:
+            # print(cmdline)
+            value = 14
+
     return value 
         
 
@@ -1273,7 +1343,7 @@ def send_sql_query_over_ssh(query:str):
     return data
 
 
-def get_light1_position(starttime:Time, endtime:Time=None, n_events:int=None, SHORT_TIME:float=10):
+def get_light1_position(starttime=None, endtime:Time=None, n_events:int=None, SHORT_TIME:float=10):
     """Give me a start and end time in astropy objects, I give you light-1 position. 
     I can also interpolate if you give me anumber of events
 
@@ -1309,13 +1379,13 @@ def get_light1_position(starttime:Time, endtime:Time=None, n_events:int=None, SH
 
     # Request the data from sql for this time period
     data =(send_sql_query_over_ssh("SELECT * FROM `LIGHT-1_Position_Data`.PositionData WHERE `Time (ModJDate)` BETWEEN " + str(starttime.to_value("mjd")) + " AND " + str(endtime.to_value("mjd")) + ";"))
+    print("SELECT * FROM `LIGHT-1_Position_Data`.PositionData WHERE `Time (ModJDate)` BETWEEN " + str(starttime.to_value("mjd")) + " AND " + str(endtime.to_value("mjd")) + ";")
     
     # Get the data from the dataframe
     latitudes   = [i for i in data['Lat (deg)']]
     longitudes  = [i for i in data['Lon (deg)']]
     times       = [i for i in data['Time (ModJDate)']]
     
-
     # Interpolation if needed
     if n_events is not None:
         times_tmp  = np.linspace(starttime.to_value(format="mjd"), endtime.to_value(format="mjd"), n_events) if n_events > 0 else np.array([(starttime.mjd+endtime.mjd)/2])
@@ -1336,4 +1406,109 @@ def get_light1_position(starttime:Time, endtime:Time=None, n_events:int=None, SH
             event_type  = "cubesat-location"
         ))
 
-    return(locs)
\ No newline at end of file
+    return(locs)
+
+def get_lightning_time_ranges(start_time, end_time):
+    """Generates a list of time ranges for each day encompassed within start_time and end_time
+
+    Args:
+        start_time (Time): start time
+        end_time (Time): end time
+    
+    Returns:
+        return_list(list): A list of time ranges. The first element is the start time, the second element is the end time, and the third element is the day corresponding to the time range. If the first element is 0, the start time is at the start of the day. If the second element is zero, the end time is the end of the day.
+    """
+    # Calculating the number of days between the given start and end times by rounding them down and subtracting start time from end time
+    rounded_start = start_time.to_datetime().replace(hour = 0, minute = 0, second = 0, microsecond = 0)
+    rounded_end = end_time.to_datetime().replace(hour = 0, minute = 0, second = 0, microsecond = 0)
+    days = (rounded_end - rounded_start).days
+
+    # Creating start and end days using the start and end times    
+    full_start_day = start_time.to_value("isot")
+    start_day = full_start_day[:4] + full_start_day[5:7] + full_start_day[8:10]
+    
+    full_end_day = end_time.to_value("isot")
+    end_day = full_end_day[:4] + full_end_day[5:7] + full_end_day[8:10]
+    
+    # Creating a list of time ranges which is to be returned
+    return_list = []
+    if days ==0:
+        # If the start and end time are within the same day, simply add the start and end times to the start day element in return_list
+        return_list.append([start_time, end_time, start_day])
+    else:
+        # If start and end time are on different days, then first of all add an element corresponding to the start time and start day
+        return_list.append([start_time, 0, start_day])
+        
+        # Create a counter variable to keep track of which day's element is being added to retrun_list
+        current_day = dt.datetime(int(start_day[0:4]), int(start_day[4:6]), int(start_day[6:]))
+
+        for i in range(days-1):
+            # For each day in between start day and end day, increment the current day by one day and add an element that corresponds to all the timestamps in that day to return_list
+            current_day = current_day + dt.timedelta(days=1)
+            temp_day = str(current_day)
+            return_list.append([0, 0, Time(temp_day)])
+        
+        # Finally add an element to return_list correspond to the end time and end day
+        return_list.append([0, end_time, end_day])
+    
+    return(return_list)
+
+def get_lightning_strikes(starttime:Time, endtime:Time):
+    """Downloads lightning strikes within the given start and end times and loads them into a rp.array
+
+    Args:
+        starttime (Time): Start time 
+        endtime (Time): End time
+
+    Raises:
+        Exception: Start time and end time must be astropy.Time objects
+
+    Returns:
+        locs (rp.array): raadpy array consisting of lightning strikes between the given start and end times
+    """
+
+    # Input Processing on start and end times
+    if (type(starttime) != Time) or (type(endtime) != Time):
+        try:
+            starttime = Time(starttime)
+            endtime = Time(endtime)
+        except:
+            raise Exception("Start time and end time must be astropy.Time objects")
+
+    # converting the give start and end times to a list of time ranges       
+    time_ranges = get_lightning_time_ranges(starttime, endtime) 
+
+    # Initialising the rp.array that will be returned
+    locs = array(event_type="en-lightning") 
+
+    # Requesting the lightning data for each time range
+    for i in tqdm(time_ranges):
+        if (i[0] != 0) & (i[1] != 0):
+            data = send_sql_query_over_ssh(f"SELECT timestamp,latitude,longitude, peakcurrent FROM LightningData.`{i[2]}` WHERE timestamp BETWEEN ' {i[0]}' AND ' {i[1]}';")
+
+        if (i[0] != 0) & (i[1] == 0):
+            data = send_sql_query_over_ssh(f"SELECT timestamp, latitude, longitude, peakcurrent FROM LightningData.`{i[2]}` WHERE timestamp >= ' {i[0]}'")
+
+        if (i[0] == 0) & (i[1] != 0):        
+            data = send_sql_query_over_ssh(f"SELECT timestamp, latitude, longitude, peakcurrent FROM LightningData.`{i[2]}` WHERE timestamp <= ' {i[1]}'")
+
+        times = [i for i in data['timestamp']]
+        latitudes = [i for i in data['latitude']]
+        longitudes = [i for i in data['longitude']]
+        currents = [i for i in data['peakcurrent']]
+
+        # Creating a rp.event object for each lightning strike and adding that to locs
+        for i in tqdm(range(len(times)),leave=False,desc=f'Day:',colour='#57068c'):
+            locs.append(event(
+                timestamp   = Time(times[i].strip(), format="isot"),
+                longitude   = longitudes[i],
+                latitude    = latitudes[i],
+                detector_id = "EN",
+                mission     = "Earth Networks",
+                time_format = "isot",
+                event_type  = "en-lightning",
+                property    = float(currents[i])
+            ))
+
+    return(locs)
+    
\ No newline at end of file
